(function ($) {

	JsBarcode = function (image, content, options, validFunction) {

		var merge = function (m1, m2) {
			var newMerge = {};
			for (var k in m1) {
				newMerge[k] = m1[k];
			}
			for (var k in m2) {
				newMerge[k] = m2[k];
			}
			return newMerge;
		};

		var validFunctionIfExist = function (valid) {
			if (validFunction) {
				validFunction(valid);
			}
		};

		options = merge(JsBarcode.defaults, options);

		var canvas = image;

		if (window.jQuery && canvas instanceof jQuery) {
			canvas = image.get(0);
		}

		if (!(canvas instanceof HTMLCanvasElement)) {
			canvas = document.createElement('canvas');
		}

		if (!canvas.getContext) {
			return image;
		}

		var encoder = new window[options.format](content);

		if (!encoder.valid()) {
			validFunctionIfExist(false);
			return this;
		}

		var binary = encoder.encoded();

		var _drawBarcodeText = function (text) {
			var x, y;

			y = options.height;

			ctx.font = options.fontSize + "px " + options.font;
			ctx.textBaseline = "bottom";
			ctx.textBaseline = 'top';

			if (options.textAlign == "left") {
				x = options.quite;
				ctx.textAlign = 'left';
			} else if (options.textAlign == "right") {
				x = canvas.width - options.quite;
				ctx.textAlign = 'right';
			} else {
				x = canvas.width / 2;
				ctx.textAlign = 'center';
			}

			ctx.fillText(text, x, y);
		};

		var ctx = canvas.getContext("2d");

		canvas.width = binary.length * options.width + 2 * options.quite;

		canvas.height = options.height + (options.displayValue ? options.fontSize * 1.3 : 0);

		ctx.clearRect(0, 0, canvas.width, canvas.height);
		if (options.backgroundColor) {
			ctx.fillStyle = options.backgroundColor;
			ctx.fillRect(0, 0, canvas.width, canvas.height);
		}

		ctx.fillStyle = options.lineColor;
		for (var i = 0; i < binary.length; i++) {
			var x = i * options.width + options.quite;
			if (binary[i] == "1") {
				ctx.fillRect(x, 0, options.width, options.height);
			}
		}

		if (options.displayValue) {
			_drawBarcodeText(content);
		}

		uri = canvas.toDataURL('image/png');

		if (window.jQuery && image instanceof jQuery) {
			if (!(image.get(0) instanceof HTMLCanvasElement)) {
				image.attr("src", uri);
			}
		} else if (!(image instanceof HTMLCanvasElement)) {
			image.setAttribute("src", uri);
		}

		validFunctionIfExist(true);
	};

	JsBarcode.defaults = {
		width: 2,
		height: 100,
		quite: 10,
		format: "CODE128",
		displayValue: false,
		font: "monospace",
		textAlign: "center",
		fontSize: 12,
		backgroundColor: "",
		lineColor: "#000"
	};

	if (window.jQuery) {
		$.fn.JsBarcode = function (content, options, validFunction) {
			JsBarcode(this, content, options, validFunction);
			return this;
		};
	}
})(window.jQuery);